@@ -26,7 +26,7 @@ check_PD <- function(vcov_list) {
 
 #' Impute a block-diagonal covariance matrix
 #'
-#' @description \loadmathjax{} \code{impute_covariance_matrix} calculates a
+#' @description \code{impute_covariance_matrix} calculates a
 #'   block-diagonal covariance matrix, given the marginal variances, the block
 #'   structure, and an assumed correlation structure. Can be used to create
 #'   compound-symmetric structures, AR(1) auto-correlated structures, or
@@ -68,39 +68,39 @@ check_PD <- function(vcov_list) {
 #' @details A block-diagonal variance-covariance matrix (possibly represented as
 #'   a list of matrices) with a specified structure. The structure depends on
 #'   whether the \code{r} argument, \code{ar1} argument, or both arguments are
-#'   specified. Let \mjeqn{v_{ij}}{v-ij} denote the specified variance for
-#'   effect \mjeqn{i}{i} in cluster \mjeqn{j}{j} and \mjeqn{C_{hij}}{C-hij} be
-#'   the covariance between effects \mjeqn{h}{h} and \mjeqn{i}{i} in cluster
-#'   \mjeqn{j}{j}. \itemize{ \item{If only \code{r} is specified,}{ each block
+#'   specified. Let \eqn{v_{ij}}{v-ij} denote the specified variance for
+#'   effect \eqn{i}{i} in cluster \eqn{j}{j} and \eqn{C_{hij}}{C-hij} be
+#'   the covariance between effects \eqn{h}{h} and \eqn{i}{i} in cluster
+#'   \eqn{j}{j}. \itemize{ \item{If only \code{r} is specified,}{ each block
 #'   of the variance-covariance matrix will have a constant (compound symmetric)
-#'   correlation, so that \mjdeqn{C_{hij} = r_j \sqrt{v_{hj} v_{ij},}}{C-hij =
-#'   r-j * sqrt(v-hj v-ij),} where \mjeqn{r_j}{r-j} is the specified correlation
-#'   for cluster \mjeqn{j}{j}. If only a single value is given in \code{r}, then
+#'   correlation, so that \deqn{C_{hij} = r_j \sqrt{v_{hj} v_{ij},}}{C-hij =
+#'   r-j * sqrt(v-hj v-ij),} where \eqn{r_j}{r-j} is the specified correlation
+#'   for cluster \eqn{j}{j}. If only a single value is given in \code{r}, then
 #'   it will be used for every cluster.} \item{If only \code{ar1} is
 #'   specified,}{ each block of the variance-covariance matrix will have an
-#'   AR(1) auto-correlation structure, so that \mjdeqn{C_{hij} = \phi_j^{|t_{hj}
+#'   AR(1) auto-correlation structure, so that \deqn{C_{hij} = \phi_j^{|t_{hj}
 #'   - t_{ij}|} \sqrt{v_{hj} v_{ij},}}{C-hij = (ar1-j)^|t-hj - t-ij| * sqrt(v-hj
-#'   v-ij),} where \mjeqn{\phi_j}{ar1-j} is the specified auto-correlation
-#'   for cluster \mjeqn{j}{j} and \mjeqn{t_{hj}}{t-hj} and \mjeqn{t_{ij}}{t-ij}
-#'   are specified time-points corresponding to effects \mjeqn{h}{h} and
-#'   \mjeqn{i}{i} in cluster \mjeqn{j}{j}. If only a single value is given in
+#'   v-ij),} where \eqn{\phi_j}{ar1-j} is the specified auto-correlation
+#'   for cluster \eqn{j}{j} and \eqn{t_{hj}}{t-hj} and \eqn{t_{ij}}{t-ij}
+#'   are specified time-points corresponding to effects \eqn{h}{h} and
+#'   \eqn{i}{i} in cluster \eqn{j}{j}. If only a single value is given in
 #'   \code{ar1}, then it will be used for every cluster.} \item{If both \code{r}
 #'   and \code{ar1} are specified,}{ each block of the variance-covariance
 #'   matrix will have combination of compound symmetric and an AR(1)
-#'   auto-correlation structures, so that \mjdeqn{C_{hij} = \left[r_j + (1 -
+#'   auto-correlation structures, so that \deqn{C_{hij} = \left[r_j + (1 -
 #'   r_j)\phi_j^{|t_{hj} - t_{ij}|}\right] \sqrt{v_{hj} v_{ij},}}{C-hij = [r-j +
 #'   (1 - r-j)(ar1-j)^|t-hj - t-ij|] * sqrt(v-hj v-ij),} where
-#'   \mjeqn{r_j}{r-j} is the specified constant correlation for cluster
-#'   \mjeqn{j}{j}, \mjeqn{\phi_j}{ar1-j} is the specified auto-correlation for
-#'   cluster \mjeqn{j}{j} and \mjeqn{t_{hj}}{t-hj} and \mjeqn{t_{ij}}{t-ij} are
-#'   specified time-points corresponding to effects \mjeqn{h}{h} and
-#'   \mjeqn{i}{i} in cluster \mjeqn{j}{j}. If only single values are given in
+#'   \eqn{r_j}{r-j} is the specified constant correlation for cluster
+#'   \eqn{j}{j}, \eqn{\phi_j}{ar1-j} is the specified auto-correlation for
+#'   cluster \eqn{j}{j} and \eqn{t_{hj}}{t-hj} and \eqn{t_{ij}}{t-ij} are
+#'   specified time-points corresponding to effects \eqn{h}{h} and
+#'   \eqn{i}{i} in cluster \eqn{j}{j}. If only single values are given in
 #'   \code{r} or \code{ar1}, they will be used for every cluster.} } If
 #'   \code{smooth_vi = TRUE}, then all of the variances within cluster
-#'   \mjeqn{j}{j} will be set equal to the average variance of cluster
-#'   \mjeqn{j}{j}, i.e., \mjdeqn{v'_{ij} = \frac{1}{n_j} \sum_{i=1}^{n_j}
+#'   \eqn{j}{j} will be set equal to the average variance of cluster
+#'   \eqn{j}{j}, i.e., \deqn{v'_{ij} = \frac{1}{n_j} \sum_{i=1}^{n_j}
 #'   v_{ij}}{v-ij' = (v-1j + ... + v-nj,j) / n-j} for
-#'   \mjeqn{i=1,...,n_j}{i=1,...,n-j} and \mjeqn{j=1,...,k}{j=1,...,k}.
+#'   \eqn{i=1,...,n_j}{i=1,...,n-j} and \eqn{j=1,...,k}{j=1,...,k}.
 #'
 #' @export
 #'
@@ -113,7 +113,6 @@ check_PD <- function(vcov_list) {
 #' MVFE <- rma.mv(d ~ 0 + test, V = V_list, data = SATcoaching)
 #' conf_int(MVFE, vcov = "CR2", cluster = SATcoaching$study)
 #' 
-#' @import mathjaxr
 
 
 impute_covariance_matrix <- function(vi, cluster, r, ti, ar1, 
@@ -184,7 +183,7 @@ impute_covariance_matrix <- function(vi, cluster, r, ti, ar1,
 
 #' Impute a patterned block-diagonal covariance matrix
 #'
-#' @description \loadmathjax{} \code{pattern_covariance_matrix} calculates a
+#' @description \code{pattern_covariance_matrix} calculates a
 #'   block-diagonal covariance matrix, given the marginal variances, the block
 #'   structure, and an assumed correlation structure defined by a patterned
 #'   correlation matrix.
@@ -207,18 +206,18 @@ impute_covariance_matrix <- function(vi, cluster, r, ti, ar1,
 #'
 #' @details A block-diagonal variance-covariance matrix (possibly represented as
 #'   a list of matrices) with a specified correlation structure, defined by a
-#'   patterned correlation matrix. Let \mjeqn{v_{ij}}{v-ij} denote the specified
-#'   variance for effect \mjeqn{i}{i} in cluster \mjeqn{j}{j} and
-#'   \mjeqn{C_{hij}}{C-hij} be the covariance between effects \mjeqn{h}{h} and
-#'   \mjeqn{i}{i} in cluster \mjeqn{j}{j}. Let \mjeqn{p_{ij}}{p-ij} be the level
-#'   of the pattern variable for effect \mjeqn{i}{i} in cluster \mjeqn{j}{j},
-#'   taking a value in \mjeqn{1,...,C}{1,...,C}. A patterned correlation matrix
+#'   patterned correlation matrix. Let \eqn{v_{ij}}{v-ij} denote the specified
+#'   variance for effect \eqn{i}{i} in cluster \eqn{j}{j} and
+#'   \eqn{C_{hij}}{C-hij} be the covariance between effects \eqn{h}{h} and
+#'   \eqn{i}{i} in cluster \eqn{j}{j}. Let \eqn{p_{ij}}{p-ij} be the level
+#'   of the pattern variable for effect \eqn{i}{i} in cluster \eqn{j}{j},
+#'   taking a value in \eqn{1,...,C}{1,...,C}. A patterned correlation matrix
 #'   is defined as a set of correlations between pairs of effects taking each
-#'   possible combination of patterns. Formally, let \mjeqn{r_{cd}}{r-cd} be the
-#'   correlation between effects in categories \mjeqn{c}{c} and \mjeqn{d}{d},
-#'   respectively, where \mjeqn{r_{cd} = r_{dc}}{r-cd = r-dc}. Then the
-#'   covariance between effects \mjeqn{h}{h} and \mjeqn{i}{i} in cluster
-#'   \mjeqn{j}{j} is taken to be \mjdeqn{C_{hij} = \sqrt{v_{hj} v_{ij}} \times
+#'   possible combination of patterns. Formally, let \eqn{r_{cd}}{r-cd} be the
+#'   correlation between effects in categories \eqn{c}{c} and \eqn{d}{d},
+#'   respectively, where \eqn{r_{cd} = r_{dc}}{r-cd = r-dc}. Then the
+#'   covariance between effects \eqn{h}{h} and \eqn{i}{i} in cluster
+#'   \eqn{j}{j} is taken to be \deqn{C_{hij} = \sqrt{v_{hj} v_{ij}} \times
 #'   r_{p_{hj} p_{ij}}.}{C-hij = sqrt(v-hj v-ij) * r[p-hj, p-ij].} 
 #'   
 #'   Correlations between effect sizes within the same category are defined by the diagonal
@@ -227,10 +226,10 @@ impute_covariance_matrix <- function(vi, cluster, r, ti, ar1,
 #'   Combinations of pattern levels that do not occur in the patterned correlation matrix will be set equal to \code{r}.
 #'   
 #'   If \code{smooth_vi = TRUE}, then all of the variances within cluster
-#'   \mjeqn{j}{j} will be set equal to the average variance of cluster
-#'   \mjeqn{j}{j}, i.e., \mjdeqn{v'_{ij} = \frac{1}{n_j} \sum_{i=1}^{n_j}
+#'   \eqn{j}{j} will be set equal to the average variance of cluster
+#'   \eqn{j}{j}, i.e., \deqn{v'_{ij} = \frac{1}{n_j} \sum_{i=1}^{n_j}
 #'   v_{ij}}{v-ij' = (v-1j + ... + v-nj,j) / n-j} for
-#'   \mjeqn{i=1,...,n_j}{i=1,...,n-j} and \mjeqn{j=1,...,k}{j=1,...,k}.
+#'   \eqn{i=1,...,n_j}{i=1,...,n-j} and \eqn{j=1,...,k}{j=1,...,k}.
 #'   
 #' @export
 #'
@@ -387,7 +386,7 @@ vcovCR.rma.mv <- function(obj, cluster, type, target, inverse_var, form = "sandw
     } 
     
     nested <- test_nested(cluster, fac = mod_struct$cluster_dat)
-    if (!all(nested)) stop("Random effects are not nested within clustering")
+    if (!all(nested)) stop("Random effects are not nested within clustering variable.")
   }
   
   if (missing(target)) {
@@ -409,21 +408,29 @@ vcovCR.rma.mv <- function(obj, cluster, type, target, inverse_var, form = "sandw
 # Get (model-based) working variance matrix 
 #-------------------------------------
 
+#' @export
+
 targetVariance.rma.mv <- function(obj, cluster) {
-  matrix_list(obj$M, cluster, "both")
+  tV <- matrix_list(obj$M, cluster, "both")
+  if (!all(sapply(tV, is.matrix))) tV <- lapply(tV, as.matrix)
+  return(tV)
 }
 
 #-------------------------------------
 # Get weighting matrix
 #-------------------------------------
 
+#' @export
+
 weightMatrix.rma.mv <- function(obj, cluster) {
   if (is.null(obj$W)) {
     V_list <- targetVariance(obj, cluster)
-    lapply(V_list, function(v) chol2inv(chol(v)))
+    Wm <- lapply(V_list, function(v) chol2inv(chol(v)))
   } else{
-    matrix_list(obj$W, cluster, "both")
+    Wm <- matrix_list(obj$W, cluster, "both")
+    if (!all(sapply(Wm, is.matrix))) Wm <- lapply(Wm, as.matrix)
   }
+  return(Wm)
 }
 
 #-----------------------------------------------
@@ -482,13 +489,36 @@ parse_structure <- function(obj) {
     mf_all <- do.call(cbind, mf_r)
     mf_s <- mf_all[obj$s.names]
     cluster_dat <- cbind(cluster_dat, mf_s)
+    cluster_dat <- droplevels(cluster_dat)
   }
   
   list(level_dat = level_dat, cluster_dat = cluster_dat)
 }
 
+#' Detect cluster structure of an rma.mv object
+#' 
+#' \code{findCluster.rma.mv} returns a vector of ID variables for the highest level of clustering in a fitted \code{rma.mv} model.
+#' 
+#' @param obj A fitted \code{rma.mv} object.
+#'   
+#' @return A a vector of ID variables for the highest level of clustering in \code{obj}.
+#'   
+#' @export
+#' 
+#' @examples
+#' library(metafor)
+#' data(hierdat, package = "robumeta")
+#' 
+#' mfor_fit <- rma.mv(effectsize ~ binge + followup + sreport + age, 
+#'                  V = var, random = list(~ 1 | esid, ~ 1 | studyid),
+#'                  data = hierdat)
+#' findCluster.rma.mv(mfor_fit)
+#' 
+
 findCluster.rma.mv <- function(obj) {
   
+  if (!inherits(obj, "rma.mv")) stop("`obj` must be a fitted rma.mv model.")
+  
   if (obj$withR) stop("vcovCR.rma.mv() does not work with fixed correlation matrices in the R argument.")
   
   # parse model structure
@@ -514,18 +544,33 @@ findCluster.rma.mv <- function(obj) {
 # Get bread matrix and scaling constant
 #---------------------------------------
 
+#' @export
+
 bread.rma.mv <- function(x, ...) {
-  if (is.null(x$W)) {
-    B <- vcov(x) * nobs(x)
-  } else{
+
+  if (inherits(x, "robust.rma")) {
+    cluster <- findCluster.rma.mv(x)
+    W <- weightMatrix(x, cluster = cluster)
     X_mat <- model_matrix(x)
-    XWX <- t(X_mat) %*% x$W %*% X_mat
-    B <- chol2inv(chol(XWX)) * nobs(x)
-    rownames(B) <- colnames(B) <- colnames(X_mat)
+    X_list <- matrix_list(X_mat, fac = cluster, dim = "row")
+    XWX_list <- Map(function(x, w) t(x) %*% w %*% x, x = X_list, w = W)
+    XWX <- Reduce(`+`, XWX_list)
+  } else {
+    if (is.null(x$W)) {
+      B <- vcov(x) * nobs(x)
+      return(B)      
+    } else {
+      X_mat <- model_matrix(x)
+      XWX <- t(X_mat) %*% x$W %*% X_mat
+    }
   }
-  B
+  B <- chol2inv(chol(XWX)) * nobs(x)
+  rownames(B) <- colnames(B) <- colnames(X_mat)
+  return(B)
 }
 
+#' @export
+
 v_scale.rma.mv <- function(obj) {
   nobs(obj)
 }
