@@ -39,7 +39,12 @@ check_CR <- function(obj, vcov, ..., tol = .Machine$double.eps^0.5) {
   # calculate E(V^CRj)  
   cluster <- attr(vcov, "cluster")
   S_array <- get_S_array(obj, vcov)
-  E_CRj <- lapply(1:nlevels(cluster), function(j) tcrossprod(S_array[,,j]))
+  if (dim(S_array)[1] == 1L) {
+    E_CRj <- lapply(1:nlevels(cluster), function(j) crossprod(S_array[1,,j]))
+  } else {
+    E_CRj <- lapply(1:nlevels(cluster), function(j) tcrossprod(S_array[,,j]))  
+  }
+  
          
   # calculate target
   Theta_list <- attr(vcov, "target")
@@ -65,30 +70,34 @@ check_CR <- function(obj, vcov, ..., tol = .Machine$double.eps^0.5) {
 
 check_sort_order <- function(obj, dat, cluster = NULL,
                              CR_types = paste0("CR",0:3),
-                             tol = 10^-6, tol2 = tol, tol3 = tol) {
+                             tol = 10^-6, tol2 = tol, tol3 = tol, 
+                             seed = NULL) {
+  
+  if (!is.null(seed)) set.seed(seed)
   
   re_order <- sample(nrow(dat))
   dat_scramble <- dat[re_order,]
   obj_scramble <- update(obj, data = dat_scramble)
 
   constraints <- utils::combn(length(coef_CS(obj)), 2, simplify = FALSE)
+  constraint_mats <- lapply(constraints, constrain_zero, coefs = coef_CS(obj))
   
   if (is.null(cluster)) {
     CR_fit <- lapply(CR_types, function(x) vcovCR(obj, type = x))
     CR_scramble <- lapply(CR_types, function(x) vcovCR(obj_scramble, type = x))
     test_fit <- lapply(CR_types, function(x) coef_test(obj, vcov = x, test = "All", p_values = FALSE))
     test_scramble <- lapply(CR_types, function(x) coef_test(obj_scramble, vcov = x, test = "All", p_values = FALSE))
-    Wald_fit <- Wald_test(obj, constraints = constraints, vcov = "CR2", test = "All")
-    Wald_scramble <- Wald_test(obj_scramble, constraints = constraints, vcov = "CR2", test = "All")
+    Wald_fit <- Wald_test(obj, constraints = constraint_mats, vcov = "CR2", test = "All")
+    Wald_scramble <- Wald_test(obj_scramble, constraints = constraint_mats, vcov = "CR2", test = "All")
     
   } else {
     CR_fit <- lapply(CR_types, function(x) vcovCR(obj, cluster = dat[[cluster]], type = x))
     CR_scramble <- lapply(CR_types, function(x) vcovCR(obj_scramble, cluster = dat_scramble[[cluster]], type = x))
     test_fit <- lapply(CR_types, function(x) coef_test(obj, vcov = x, cluster = dat[[cluster]], test = "All", p_values = FALSE))
     test_scramble <- lapply(CR_types, function(x) coef_test(obj_scramble, vcov = x, cluster = dat_scramble[[cluster]], test = "All", p_values = FALSE))
-    Wald_fit <- Wald_test(obj, constraints = constraints, vcov = "CR2",
+    Wald_fit <- Wald_test(obj, constraints = constraint_mats, vcov = "CR2",
                           cluster = dat[[cluster]], test = "All")
-    Wald_scramble <- Wald_test(obj_scramble, constraints = constraints, vcov = "CR2", 
+    Wald_scramble <- Wald_test(obj_scramble, constraints = constraint_mats, vcov = "CR2", 
                                cluster = dat_scramble[[cluster]], test = "All")
     
   }
@@ -109,14 +118,12 @@ compare_ttests <- function(a, b, tol = 10^-6) {
   testthat::expect_equal(a$saddlepoint, b$saddlepoint, tolerance = tol)
 }
 
-compare_Waldtests <- function(a, b, tol = 10^-6) {
+compare_Waldtests <- function(a, b, tol = 10^-4) {
   
   if (!inherits(a,"data.frame")) a <- do.call(rbind, a)
   if (!inherits(b,"data.frame")) b <- do.call(rbind, b)
   
   testthat::expect_equal(a$Fstat, b$Fstat, tolerance = tol)
-  testthat::expect_equal(a$delta, b$delta, tolerance = tol)
   testthat::expect_equal(a$df, b$df, tolerance = tol)
-  testthat::expect_equal(a$p_val, b$p_val, tolerance = tol)
   
 }
\ No newline at end of file
